﻿using System;
using System.Security.Cryptography;
using System.Text;
using Microsoft.Extensions.DependencyInjection;
using NServiceBus;

namespace CareVisit.EventIntegration.IoC
{
    public static class NServiceBusExtensions
    {
        const int MaxEntityName = 50;

        public static IServiceCollection RegisterNServiceBus(this IServiceCollection services)
        {
            IEndpointInstance endpointInstance = null;

            var endpointConfiguration = new EndpointConfiguration("CareVisit.EventIntegration");
            endpointConfiguration.EnableInstallers();
            endpointConfiguration.UseSerialization<NewtonsoftSerializer>();
            endpointConfiguration.UsePersistence<InMemoryPersistence>();

            endpointConfiguration.SetDiagnosticsPath(@"C:\TEMP\");

            endpointConfiguration.SendFailedMessagesTo("error");
            endpointConfiguration.AuditProcessedMessagesTo("audit");

            endpointConfiguration.Conventions().DefiningEventsAs(t => t.Namespace != null && t.Namespace.EndsWith("Events"));

            var transport = endpointConfiguration.UseTransport<AzureServiceBusTransport>();
            transport.SubscriptionNameShortener(n => n.Length > MaxEntityName ? MD5DeterministicNameBuilder.Build(n) : n);
            transport.RuleNameShortener(n => n.Length > MaxEntityName ? MD5DeterministicNameBuilder.Build(n) : n);
            transport.ConnectionString("RemoveIntentionally");

            services.AddSingleton<IMessageSession>(sp => endpointInstance);

            endpointConfiguration.UseContainer<ServicesBuilder>(c =>
            {
                c.ExistingServices(services);
            });

            endpointInstance = Endpoint.Start(endpointConfiguration).GetAwaiter().GetResult();

            return services;
        }

        static class MD5DeterministicNameBuilder
        {
            public static string Build(string input)
            {
                var inputBytes = Encoding.Default.GetBytes(input);
                // use MD5 hash to get a 16-byte hash of the string
                using (var provider = new MD5CryptoServiceProvider())
                {
                    var hashBytes = provider.ComputeHash(inputBytes);
                    return new Guid(hashBytes).ToString();
                }
            }
        }
    }
}